#include "MotorCtl.h"
#include "Arduino.h"

// Motor control arrays
uint16_t a1Pins[2];
uint16_t a2Pins[2];
uint16_t pwmPins[2];


//*************************************************************************
// Motor control initialization
//*************************************************************************
void motorInit(uint16_t leftA1,  uint16_t leftA2,  uint16_t leftPwm,
               uint16_t rightA1, uint16_t rightA2, uint16_t rightPwm)
{

    // Motor control arrays
    // Weird criss cross in these definitions is because to go forward, one
    // motor has to go CW and the other CCW
    a1Pins[MOTOR_LT] = leftA1;  
    a1Pins[MOTOR_RT] = rightA2;
    a2Pins[MOTOR_LT] = leftA2;
    a2Pins[MOTOR_RT] = rightA1;
    pwmPins[MOTOR_LT] = leftPwm;
    pwmPins[MOTOR_RT] = rightPwm;

    // Motor controller pins
    pinMode(leftA2,  OUTPUT);
    pinMode(leftA1,  OUTPUT);
    pinMode(rightA2, OUTPUT);
    pinMode(rightA1, OUTPUT);
    
}


//*************************************************************************
// Set motors to go forward
//*************************************************************************
void motorsFwd(void)
{
  motorCw(MOTOR_RT);
  motorCcw(MOTOR_LT);
}


//*************************************************************************
// Set motors to go in reverse
//*************************************************************************
void motorsRev(void)
{
  motorCw(MOTOR_LT);
  motorCcw(MOTOR_RT);
}

//*************************************************************************
// Set motors to brake
//*************************************************************************
void motorsBrake(void)
{
  motorBrake(MOTOR_LT);
  motorBrake(MOTOR_RT);
}


//*************************************************************************
// Set the pins for CCW motion on a given motor
//*************************************************************************
void motorCcw(Motor_e id)
{
  digitalWrite(a1Pins[id], 0);
  digitalWrite(a2Pins[id], 1);
}

//*************************************************************************
// Set the pins for CW motion on a given motor
//*************************************************************************
void motorCw(Motor_e id)
{
  digitalWrite(a1Pins[id], 1);
  digitalWrite(a2Pins[id], 0);
}

//*************************************************************************
// Set the pins for brake action on a given motor
//*************************************************************************
void motorBrake(Motor_e id)
{
  digitalWrite(a1Pins[id], 1);
  digitalWrite(a2Pins[id], 1);
}

//*************************************************************************
// Set the pins for float action on a given motor
//*************************************************************************
void motorFloat(Motor_e id)
{
  digitalWrite(a1Pins[id], 0);
  digitalWrite(a2Pins[id], 0);
}

//*************************************************************************
// Set the motor speed
//*************************************************************************
void motorSpeed(Motor_e id, int16_t spd)
{
  
  // Dead band on motor so it does not hum with small speeds
  if (spd < 20) spd = 0.0;

  // Limit to max PWM
  if (spd > 255) spd = 255;
  
  // Set up the PWM
  analogWrite(pwmPins[id], spd);         
}

