#include "Range.h"
#include "Arduino.h"


typedef struct
{
  float volts;
  float range;
} Entry_t;

// Table of voltages and corresponding distance of the IR sensor.
// Taken from the datasheet
static Entry_t table[] = 
{
  //volts, cm
  {3.2,  2.0},
  {2.0,  4.0},
  {1.2,  6.0},
  {0.9,  8.0},
  {0.6,  10.0},
  {0.5,  12.0},
  {0.35, 14.0},
  {0.3,  16.0},
  {0.2,  18.0},
  {0.05, 20.0}
};

// Number of entries in the lookup table
#define N  (sizeof(table)/sizeof(Entry_t))

static int sensorPin;

static float lookup(float volt);



void rangeInit(int pin)
{
  sensorPin = pin;
}

//*************************************************************************
// Determine the range given the measured voltage
//*************************************************************************
static float lookup(float volt)
{
  int   i;
  float m, b;
  
  for (i=0; i<N-1; i++) 
  {
    if ( (volt <= table[i].volts) && (volt >= table[i+1].volts) )
    {
      m = (table[i+1].range - table[i].range) / (table[i+1].volts - table[i].volts);
      b = table[i].range - m * table[i].volts;
      return  (volt * m + b);
    }
  }
  return 0.0;
}

//*************************************************************************
// Read range sensor 
//*************************************************************************
float rangeRead(void)
{
  
  float volt;
  float range;
  
  // Read voltage from sensor
  volt = (float)analogRead(sensorPin) * 5.0 / 1024.0;

  // Convert voltage to range in cm
  range = lookup(volt);

  return range;

}


